"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateExtensions = void 0;
const node_path_1 = __importDefault(require("node:path"));
const core_1 = require("@graphql-codegen/core");
const scalars_1 = require("@aws-amplify/appsync-modelgen-plugin/lib/scalars");
const plugin_1 = require("./plugin");
async function generateExtensions(options) {
    const { schema, directives, 
    // feature flags
    generateIndexRules = true, emitAuthProvider = true, useExperimentalPipelinedTranformer = true, transformerVersion = 2, respectPrimaryKeyAttributesOnConnectionField = true, generateModelsForLazyLoadAndCustomSelectionSet = true, addTimestampFields = true, handleListNullabilityTransparently = true, } = options;
    const codegenOptions = {
        filename: node_path_1.default.join(options.baseOutputDir, "DataStoreExtension.dart"),
        plugins: [
            {
                appSyncDartExtensionPlugin: {},
            },
        ],
        schema,
        documents: [],
        config: {
            directives,
            scalars: { ...scalars_1.DART_SCALAR_MAP },
            target: "dart",
            isTimestampFieldsAdded: addTimestampFields,
            emitAuthProvider,
            generateIndexRules,
            handleListNullabilityTransparently,
            usePipelinedTransformer: useExperimentalPipelinedTranformer,
            transformerVersion,
            respectPrimaryKeyAttributesOnConnectionField,
            generateModelsForLazyLoadAndCustomSelectionSet,
        },
        pluginMap: {
            appSyncDartExtensionPlugin: {
                plugin: plugin_1.plugin,
                addToSchema: plugin_1.addToSchema,
            },
        },
    };
    const generatedCode = await (0, core_1.codegen)(codegenOptions);
    return { [codegenOptions.filename]: generatedCode };
}
exports.generateExtensions = generateExtensions;
//# sourceMappingURL=extension.js.map