"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readNumericFeatureFlag = exports.readFeatureFlag = void 0;
const amplify_cli_core_1 = require("amplify-cli-core");
/**
 * Return feature flag override values from the cli in the format --feature-flag:<feature flag name> <feature flag value>
 * @param {!AmplifyContext} context the amplify runtime context
 * @param {!string} flagName the feature flag name
 * @returns {any | null} the raw value if found, else null
 */
const cliFeatureFlagOverride = (context, flagName) => { var _a, _b; return (_b = (_a = context.parameters) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b[`feature-flag:${flagName}`]; };
/**
 * Returns feature flag value, default to `false`
 * @param {!AmplifyContext} context the amplify runtime context
 * @param {!string} flagName feature flag id
 * @returns {!boolean} the feature flag value
 */
const readFeatureFlag = (context, flagName) => {
    const cliValue = cliFeatureFlagOverride(context, flagName);
    if (cliValue) {
        if (cliValue === "true" || cliValue === "True" || cliValue === true) {
            return true;
        }
        if (cliValue === "false" || cliValue === "False" || cliValue === false) {
            return false;
        }
        throw new Error(`Feature flag value for parameter ${flagName} could not be marshalled to boolean type, found ${cliValue}`);
    }
    try {
        return amplify_cli_core_1.FeatureFlags.getBoolean(flagName);
    }
    catch (_) {
        return false;
    }
};
exports.readFeatureFlag = readFeatureFlag;
/**
 * Returns feature flag value, default to `1`
 * @param {!AmplifyContext} context the amplify runtime context
 * @param {!string} flagName feature flag id
 * @returns {!number} the feature flag value
 */
const readNumericFeatureFlag = (context, flagName) => {
    const cliValue = cliFeatureFlagOverride(context, flagName);
    if (cliValue) {
        return Number.parseInt(cliValue, 10);
    }
    try {
        return amplify_cli_core_1.FeatureFlags.getNumber(flagName);
    }
    catch (_) {
        return 1;
    }
};
exports.readNumericFeatureFlag = readNumericFeatureFlag;
//# sourceMappingURL=feature-flags.js.map