"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppSyncDartExtensionVisitor = void 0;
const appsync_visitor_1 = require("@aws-amplify/appsync-modelgen-plugin/lib/visitors/appsync-visitor");
const dart_declaration_block_1 = require("@aws-amplify/appsync-modelgen-plugin/lib/languages/dart-declaration-block");
const scalars_1 = require("@aws-amplify/appsync-modelgen-plugin/lib/scalars");
const dart_config_1 = require("@aws-amplify/appsync-modelgen-plugin/lib/configs/dart-config");
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
class AppSyncDartExtensionVisitor extends appsync_visitor_1.AppSyncModelVisitor {
    constructor(schema, rawConfig, additionalConfig, defaultScalars = scalars_1.DART_SCALAR_MAP) {
        super(schema, rawConfig, additionalConfig, defaultScalars);
    }
    generate() {
        // TODO: Remove us, leaving in to be explicit on why this flag is here.
        const shouldUseModelNameFieldInHasManyAndBelongsTo = true;
        // This flag is going to be used to tight-trigger on JS implementations only.
        const shouldImputeKeyForUniDirectionalHasMany = false;
        this.processDirectives(shouldUseModelNameFieldInHasManyAndBelongsTo, shouldImputeKeyForUniDirectionalHasMany);
        return this.generateModelExtensionClasses();
    }
    /**
     * Generate classes with model directives
     */
    generateModelExtensionClasses() {
        const result = [];
        //Custom lints warning
        result.push(dart_config_1.CUSTOM_LINTS_MESSAGE);
        //Ignore for file
        result.push(dart_config_1.IGNORE_FOR_FILE);
        //Imports
        const packageImports = this.generatePackageHeader();
        result.push(packageImports);
        //Extension
        Object.entries(this.getSelectedModels()).forEach(([, model]) => {
            const extensionDeclaration = this.generateExtensionClass(model);
            result.push(extensionDeclaration);
        });
        return result.join("\n\n");
    }
    generateExtensionClass(model) {
        const modelName = this.getModelName(model);
        const classDeclarationBlock = new dart_declaration_block_1.DartDeclarationBlock();
        classDeclarationBlock
            .asKind("extension")
            .extensionOn("AmplifyDataStore")
            .withName(`${modelName}Extension`)
            .withComment(`This is an auto generated extension representing the ${modelName} type in your schema.`);
        // get method
        this.generateGetModelMethod(model, classDeclarationBlock);
        this.generateGetModelMethod(model, classDeclarationBlock, true);
        return classDeclarationBlock.string;
    }
    generateGetModelMethod(model, classDeclarationBlock, nullable = false) {
        const identifierFields = this.getModelIdentifierFields(model);
        const args = identifierFields.map((field) => ({
            name: field.name,
            type: this.getNativeType(field),
        }));
        const identifierArgs = args
            .map((arg) => `${arg.name}: ${arg.name}`)
            .join(", ");
        const modelName = this.getModelName(model);
        const where = this.config.respectPrimaryKeyAttributesOnConnectionField
            ? `${modelName}.MODEL_IDENTIFIER.eq(${modelName}ModelIdentifier(${identifierArgs})),`
            : `${modelName}.ID.eq(${identifierArgs}),`;
        classDeclarationBlock.addClassMethod(`get${modelName}${nullable ? "OrNull" : ""}`, `Future<${modelName}${nullable ? "?" : ""}>`, args, [
            "return query(",
            (0, visitor_plugin_common_1.indent)(`${modelName}.classType,`),
            (0, visitor_plugin_common_1.indent)(`where: ${where}`),
            ")",
            `.then((list) => list.${nullable ? "singleOrNull" : "single"});`,
        ].join("\n"), undefined);
    }
    getModelIdentifierFields(model) {
        if (this.selectedTypeIsNonModel()) {
            return [];
        }
        if (!this.config.respectPrimaryKeyAttributesOnConnectionField) {
            return [model.fields.find((f) => f.name === "id")];
        }
        const primaryKeyField = this.getModelPrimaryKeyField(model);
        const { sortKeyFields } = primaryKeyField.primaryKeyInfo;
        return [primaryKeyField, ...sortKeyFields];
    }
    generatePackageHeader() {
        let usingCollection = false;
        Object.entries({
            ...this.getSelectedModels(),
            ...this.getSelectedNonModels(),
        }).forEach(([, model]) => {
            model.fields.forEach((f) => {
                if (f.isList) {
                    usingCollection = true;
                }
            });
        });
        const flutterDatastorePackage = dart_config_1.FLUTTER_AMPLIFY_CORE_IMPORT;
        const packagesImports = [
            usingCollection ? dart_config_1.COLLECTION_PACKAGE : "",
            `${dart_config_1.LOADER_CLASS_NAME}.dart`,
        ]
            .filter((f) => f)
            .map((pckg) => `import '${pckg}';`);
        packagesImports.push(`import '${flutterDatastorePackage}.dart' as ${dart_config_1.AMPLIFY_CORE_PREFIX};`);
        packagesImports.push(`import 'package:amplify_datastore/amplify_datastore.dart';`);
        return packagesImports.sort().join("\n") + "\n";
    }
}
exports.AppSyncDartExtensionVisitor = AppSyncDartExtensionVisitor;
//# sourceMappingURL=appsync-dart-extension-visitor.js.map